# Generated by Django 3.1 on 2020-10-09 00:00

from django.db import migrations, models
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0009_auto_20201001_1913'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='category',
            field=models.CharField(max_length=120, verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='question',
            name='answer',
            field=models.CharField(max_length=255, verbose_name='Answer'),
        ),
        migrations.AlterField(
            model_name='question',
            name='question',
            field=models.CharField(max_length=120, verbose_name='Question'),
        ),
        migrations.AlterField(
            model_name='team',
            name='categories',
            field=multiselectfield.db.fields.MultiSelectField(choices=[(1, 'Comment'), (2, 'Sales'), (3, 'Help'), (4, 'Bug')], max_length=7, verbose_name='Categories'),
        ),
        migrations.AlterField(
            model_name='team',
            name='name',
            field=models.CharField(max_length=30, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='dev_ticket',
            field=models.CharField(blank=True, max_length=30, null=True, verbose_name='Developer Ticket'),
        ),
    ]



# Generated by Django 3.1 on 2020-09-01 23:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0003_auto_20200901_2025'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='sites',
            field=models.ManyToManyField(blank=True, related_name='categories', to='sites.Site'),
        ),
    ]



from django.views.generic import TemplateView
from django.views.generic.edit import CreateView
from django.views.generic.list import ListView
from django.urls import reverse_lazy
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.models import Site

from helpme.models import Ticket, Comment, Team, Question, Category, VisibilityChoices, StatusChoices
from helpme.forms import TicketForm, CommentForm


class FAQView(LoginRequiredMixin, TemplateView):
    template_name = "helpme/faq.html"
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        current_site = Site.objects.get_current()
        categories = Category.objects.filter(category_sites__in=[current_site])
        context['categories'] = categories
        if not categories.exists():
            context['questions'] = Question.objects.filter(sites__in=[current_site])
        return context


class SupportRequestView(LoginRequiredMixin, CreateView):
    model = Ticket
    success_url = reverse_lazy('helpme:dashboard')
    form_class = TicketForm

    def form_valid(self, form):
        form.instance.user = self.request.user
        form.instance.site = Site.objects.get_current()

        user_agent = self.request.user_agent

        if user_agent.is_mobile:
            device = "mobile"
        elif user_agent.is_pc:
            device = "pc"
        elif user_agent.is_tablet:
            device = "tablet"
        else:
            device = "unknown"

        form.instance.user_meta = {
            "browser": {
                "family": str.lower(user_agent.browser.family),
                "version": user_agent.browser.version_string
            },
            "os": {
                "family": str.lower(user_agent.os.family),
                "version": user_agent.os.version_string
            },
            "device": str.lower(user_agent.device.family),
            "mobile_tablet_or_pc": device,
            "ip_address": self.request.META['REMOTE_ADDR']
        }

        response = super().form_valid(form)

        # filter and assign teams by site and category
        teams = Team.objects.filter(sites__in=[form.instance.site])
        form.instance.teams.set(teams.filter(categories__contains=form.instance.category))

        return response


class SupportDashboardView(LoginRequiredMixin, ListView):
    model = Ticket
    paginate_by = 10

    def get_paginate_by(self, queryset):
        if 'paginate_by' in self.request.GET:
            # returning the value doesn't work, have to explicitly set it
            self.paginate_by = int(self.request.GET['paginate_by']) # force to int just in case there's mischief

        return self.paginate_by

    def get_queryset(self, **kwargs):
        # admin
        if self.request.user.has_perm('helpme.see_all_tickets'):
            queryset = Ticket.objects.all()
        # support team member
        # sees tickets that are assigned to them or to a team they belong to
        # but are not assigned to a specific user yet
        elif self.request.user.has_perm('helpme.see_support_tickets'):
            tickets = Ticket.objects.filter(assigned_to=self.request.user) | Ticket.objects.filter(teams__in=self.request.user.team_set.all(), assigned_to=None)
            queryset = tickets.distinct()
        # platform user
        else:
            queryset = Ticket.objects.filter(user=self.request.user)

        # filter by status
        s = self.request.GET.get('s', '')
        if s:
            status = s.split(',')
            for stat in status:
                stat = int(stat)
            queryset = queryset.filter(status__in=status)
        else:
            # exclude closed tickets by default
            queryset = queryset.exclude(status=StatusChoices.CLOSED)
            
        return queryset.order_by('-priority')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        support = self.request.user.has_perm('helpme.see_support_tickets')
        developer = self.request.user.has_perm('helpme.see_developer_tickets')
        admin = self.request.user.has_perm('helpme.see_all_tickets')
        
        context['support'] = support
        context['statuses'] = StatusChoices.choices
        context['s'] = self.request.GET.get('s', '')
        context['pagination'] = self.paginate_by
        
        # determines whether status color is red or green
        context['negative_status'] = [StatusChoices.CLOSED, StatusChoices.CANCELED, StatusChoices.HOLD]
        
        context['ticket_form'] = TicketForm
        context['comment_form'] = CommentForm(support=support)

        # filter comments by visibility
        if admin:
            comments = Comment.objects.all()
        else:
            comments = Comment.objects.filter(visibility=VisibilityChoices.REPORTERS)
            if support:
                comments |=  Comment.objects.filter(visibility=VisibilityChoices.SUPPORT)
            if developer:
                comments |= Comment.objects.filter(visibility=VisibilityChoices.DEVELOPERS)
        context['comments'] = comments
        return context
    



from django import template
from datetime import datetime

from helpme.models import CommentTypeChoices 

register = template.Library()

@register.filter
def sort_by(queryset, order):
    return queryset.order_by(order)

# find the ticket's last message visible to the current user
@register.filter
def last_visible(ticket_comments, comments):
    last_visible = None
    for comment in ticket_comments.filter(comment_type=CommentTypeChoices.MESSAGE).order_by('-created'):
        if comment in comments:
            last_visible = comment
            break
    return last_visible

@register.filter
def convert_isotime(isotime):
    return datetime.strptime(isotime[:19], '%Y-%m-%dT%H:%M:%S')

@register.simple_tag
def url_replace(request, field, value):

    dict_ = request.GET.copy()

    dict_[field] = value

    return dict_.urlencode()

@register.simple_tag
def multiply(a, b):
    return a * b




from rest_framework.generics import CreateAPIView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.models import Site
from django.shortcuts import redirect

from helpme.models import Ticket, Comment, Category, Question, Team
from helpme.api.serializers import TicketSerializer, CommentSerializer, CategorySerializer, QuestionSerializer


class CreateTicketAPIView(LoginRequiredMixin, CreateAPIView):
    serializer_class = TicketSerializer
    queryset = Ticket.objects.all()

    def perform_create(self, serializer):
        user_agent = self.request.user_agent
        
        if user_agent.is_mobile:
            device = "mobile"
        elif user_agent.is_pc:
            device = "pc"
        elif user_agent.is_tablet:
            device = "tablet"
        else:
            device = "unknown"

        user_meta = {
            "browser": {
                "family": str.lower(user_agent.browser.family),
                "version": user_agent.browser.version_string
            },
            "os": {
                "family": str.lower(user_agent.os.family),
                "version": user_agent.os.version_string
            },
            "device": str.lower(user_agent.device.family),
            "mobile_tablet_or_pc": device,
            "ip_address": self.request.META['REMOTE_ADDR']
        }
        
        instance = serializer.save(user=self.request.user, site=Site.objects.get_current(), user_meta=user_meta)

        # filter and assign teams by site and category
        teams = Team.objects.filter(sites__in=[instance.site])
        instance.teams.set(teams.filter(categories__contains=instance.category))

    def create(self, request, *args, **kwargs):
        response = super().create(request, *args, **kwargs)
        if request.user.has_perm('helpme.see_support_tickets'):
            return redirect('helpme_admin:dashboard')
        return redirect('helpme:dashboard')
    

class CreateCommentAPIView(LoginRequiredMixin, CreateAPIView):
    lookup_field = 'uuid'
    serializer_class = CommentSerializer
    queryset = Ticket.objects.all()

    def perform_create(self, serializer):
        ticket = self.get_object()
        instance = serializer.save(user=self.request.user, ticket=ticket)

    def create(self, request, *args, **kwargs):
        response = super().create(request, *args, **kwargs)
        uuid = self.kwargs.get("uuid")
        if request.user.has_perm('helpme.see_support_tickets'):
            return redirect('helpme_admin:ticket-detail', uuid=uuid)
        else:
            return redirect('helpme:dashboard')


class CreateCategoryAPIView(LoginRequiredMixin, CreateAPIView):
    serializer_class = CategorySerializer
    queryset = Category.objects.all()

    def perform_create(self, serializer):
        instance = serializer.save()
        if instance.global_category:
            instance.category_sites.set(Site.objects.exclude(id__in=instance.category_excluded_sites.all()))
        elif not instance.category_sites.exists():
            instance.category_sites.add(Site.objects.get_current())

    def create(self, request, *args, **kwargs):
        response = super().create(request, *args, **kwargs)
        return redirect('helpme_admin:faq-create')


class CreateQuestionAPIView(LoginRequiredMixin, CreateAPIView):
    serializer_class = QuestionSerializer
    queryset = Question.objects.all()

    def perform_create(self, serializer):
        instance = serializer.save()
        if instance.global_question:
            instance.sites.set(Site.objects.exclude(id__in=instance.excluded_sites.all()))
        elif not instance.sites.exists():
            instance.sites.add(Site.objects.get_current())

    def create(self, request, *args, **kwargs):
        response = super().create(request, *args, **kwargs)
        return redirect('helpme_admin:faq-create')



from django.views.generic.edit import CreateView, UpdateView
from django.urls import reverse_lazy
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.contrib.sites.models import Site
from django.template.loader import render_to_string

from helpme.models import Ticket, Comment, Team, VisibilityChoices, StatusChoices, CommentTypeChoices
from helpme.forms import TicketForm, UpdateTicketForm, CommentForm, QuestionForm, CategoryForm, TeamForm
from .helpme import FAQView, SupportDashboardView


class FAQCreateView(PermissionRequiredMixin, FAQView):
    template_name = "helpme/faq_create.html"
    permission_required = ["helpme.view_question", "helpme.view_category"]
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['question_form'] = QuestionForm
        context['category_form'] = CategoryForm
        return context
    

class AdminSupportDashboardView(PermissionRequiredMixin, SupportDashboardView):
    permission_required = "helpme.see_support_tickets"
    template_name = "helpme/admin_ticket_list.html"
    

class TicketDetailView(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    model = Ticket
    form_class = UpdateTicketForm
    template_name = "helpme/ticket_detail.html"
    permission_required = "helpme.see_support_tickets"

    def get_success_url(self):
        return reverse_lazy('helpme_admin:ticket-detail', args=[self.object.uuid])
    
    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj

    def sort_comments(self):
        ticket = self.get_object()
        if 'oldest' in self.request.GET:
            return ticket.comments.all().order_by('created')
        else:
            return ticket.comments.all().order_by('-created')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        support = self.request.user.has_perm('helpme.see_support_tickets')
        developer = self.request.user.has_perm('helpme.see_developer_tickets')
        admin = self.request.user.has_perm('helpme.see_all_tickets')
        
        context['support'] = support
        context['developer'] = developer
        context['admin'] = admin

        context['user'] = self.request.user
        context['comment_form'] = CommentForm(support=support)
        context['ticket_form'] = TicketForm
        
        # determines whether status color is red or green
        context['negative_status'] = [StatusChoices.CLOSED, StatusChoices.CANCELED, StatusChoices.HOLD]

        # filter comments by visibility
        if admin:
            comments = self.object.comments.all()
        else:
            comments = self.object.comments.filter(visibility=VisibilityChoices.REPORTERS)
            if support:
                comments |=  self.object.comments.filter(visibility=VisibilityChoices.SUPPORT)
            if developer:
                comments |= self.object.comments.filter(visibility=VisibilityChoices.DEVELOPERS)
        context['comments'] = comments
        context['ticket_comments'] = self.sort_comments
        return context

    def form_valid(self, form):
        context = {
            "changed_fields": form.changed_data,
            "user": self.request.user.username,
            "multiple": len(form.changed_data) > 1
        }
        action = render_to_string('helpme/event_comment.txt', context)
        Comment.objects.create(content=action, user=self.request.user, ticket=self.get_object(), comment_type=CommentTypeChoices.EVENT, visibility=VisibilityChoices.SUPPORT)
        
        response = super().form_valid(form)
        return response


class TeamCreateView(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'helpme.view_team'
    model = Team
    form_class = TeamForm
    success_url = reverse_lazy('helpme_admin:team-list')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['admin'] = self.request.user.has_perm('helpme.add_team')
        context['user_teams'] = self.request.user.team_set.all()
        context['teams'] = Team.objects.filter(sites__in=[Site.objects.get_current()])
        return context
    

    def form_valid(self, form):
        response = super().form_valid(form)
        form.instance.sites.add(Site.objects.get_current())
        return response
    
    
class TeamDetailView(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'helpme.view_team'
    model = Team
    form_class = TeamForm
    template_name = "helpme/team_detail.html"

    def get_success_url(self):
        return reverse_lazy('helpme_admin:team-detail', args=[self.object.uuid])

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        uuid = self.kwargs.get('uuid')

        try:
            obj = queryset.get(uuid=uuid)
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['admin'] = self.request.user.has_perm('helpme.change_team')
        return context



from django.test import TestCase, Client
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.sites.models import Site
from django.urls import reverse

from .models import Ticket, Comment, Team, Category, Question, StatusChoices, PriorityChoices, VisibilityChoices, CommentTypeChoices
from .settings import app_settings

class ModelTests(TestCase):

    def test_create_ticket_model(self):
        ticket = Ticket.objects.create(subject="Test", description="Test description", category=app_settings.TICKET_CATEGORIES.HELP)
        
        self.assertEqual(ticket.site.domain, "example.com")
        self.assertEqual(ticket.status, StatusChoices.OPEN)
        self.assertEqual(ticket.priority, PriorityChoices.MEDIUM)

        
    def test_create_comment_model(self):
        ticket = Ticket.objects.create(subject="Test", description="Test description", category=app_settings.TICKET_CATEGORIES.HELP)
        comment = Comment.objects.create(content="This is an example comment", ticket=ticket)
    
        self.assertEqual(comment.visibility, VisibilityChoices.REPORTERS)
        self.assertEqual(comment.comment_type, CommentTypeChoices.MESSAGE)

        
class ClientTests(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.client = Client()
        cls.support = get_user_model().objects.create(username="supportuser")
        cls.student = get_user_model().objects.create(username="testuser")
        cls.admin = get_user_model().objects.create_superuser(username="admin")
        
        support = Permission.objects.get(codename="see_support_tickets")
        view_team = Permission.objects.get(codename="view_team")
        view_category = Permission.objects.get(codename="view_category")
        view_question = Permission.objects.get(codename="view_question")
        add_category = Permission.objects.get(codename="add_category")
        add_question = Permission.objects.get(codename="add_question")
        cls.support.user_permissions.add(support, view_team, add_category, add_question, view_category, view_question)
        
        cls.support_team = Team.objects.create(name="Support", global_team=False, categories=str(app_settings.TICKET_CATEGORIES.HELP))
        site = Site.objects.get(pk=1)
        cls.support_team.sites.add(site)
        cls.support_team.members.add(cls.support)

        cls.basic_category = Category.objects.create(category="Testing")
        cls.basic_category.category_sites.add(site)
        cls.question = Question.objects.create(question="What is 2 + 2?", answer="4", category=cls.basic_category)

        
    def setUp(self):
        self.simple_ticket = Ticket.objects.create(subject="ABC", description="123", category=app_settings.TICKET_CATEGORIES.COMMENT, user=self.student)
        self.simple_ticket.teams.add(self.support_team)
        self.client.force_login(self.support)

        
    def test_create_ticket(self):
        uri = reverse("helpme-api-create-ticket")
        self.assertEqual(Ticket.objects.all().count(), 1)
        
        response = self.client.post(uri, {"subject": "Test subject", "description": "Test description", "category": app_settings.TICKET_CATEGORIES.HELP}, follow=True)
        self.assertEqual(Ticket.objects.all().count(), 2)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(reverse("helpme_admin:dashboard"), 302)])
        ticket = Ticket.objects.get(subject="Test subject")
        self.assertEqual(ticket.user, self.support)
        self.assertQuerysetEqual(ticket.teams.all(), ['<Team: Support>'])
        self.assertEqual(ticket.user_meta, {'os': {'family': 'other', 'version': ''}, 'device': 'other', 'browser': {'family': 'other', 'version': ''}, 'ip_address': '127.0.0.1', 'mobile_tablet_or_pc': 'unknown'})

        
    def test_dashboard_support(self):
        uri = reverse("helpme_admin:dashboard")
        response = self.client.get(uri)
        self.assertEqual(self.support.has_perm("helpme.see_support_tickets"), True)
        self.assertContains(response, reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid]))
        self.assertContains(response, 'testuser - ABC')
        self.assertContains(response, "Status:")
        self.assertContains(response, "Open")


    def test_dashboard_student(self):
        self.client.force_login(self.student)
        uri = reverse("helpme:dashboard")
        response = self.client.get(uri)
        self.assertEqual(self.student.has_perm("helpme.see_support_tickets"), False)
        self.assertContains(response, 'ABC')
        self.assertContains(response, "Status: ")
        self.assertContains(response, "Open")


    def test_dashboard_filter_by_status(self):
        active_ticket = Ticket.objects.create(subject="Active ticket", description="Test filter", category=app_settings.TICKET_CATEGORIES.HELP, status=StatusChoices.ACTIVE, user=self.support)
        active_ticket.teams.add(self.support_team)
        
        # filter by active tickets
        uri = reverse("helpme_admin:dashboard")
        response = self.client.get(uri, {'s': '10'})

        self.assertContains(response, 'supportuser - Active ticket')
        self.assertNotContains(response, 'testuser - ABC')

        # filter by active and open tickets
        new_response = self.client.get(uri, {"s": "1,10"})
        self.assertContains(new_response, 'supportuser - Active ticket')
        self.assertContains(new_response, 'testuser - ABC')

        
    def test_ticket_detail_support(self):
        uri = reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid])
        response = self.client.get(uri)

        # ticket information and update form
        self.assertContains(response, "ABC")
        self.assertContains(response, "123")
        self.assertContains(response, '<select name="status"')
        self.assertContains(response, '<option value="3" selected>Medium</option>')
        self.assertContains(response, '<option value="1" selected>Comment</option>')
        self.assertContains(response, 'checked="checked" name="teams"')
        self.assertContains(response, '<select name="assigned_to"')
        self.assertContains(response, '<input type="text" name="dev_ticket"')
        self.assertContains(response, "Reporter:")
        self.assertContains(response, "Reporter Meta")
        self.assertContains(response, "Site:")
        self.assertContains(response, "Created:")
        self.assertContains(response, "Last Updated:")
        self.assertContains(response, '<input type="submit" value="Update"')

        # comment form
        self.assertContains(response, '<textarea name="content"')
        self.assertContains(response, '<select name="visibility"')
        self.assertContains(response, '<input type="submit" value="Reply"')


    def test_ticket_detail_student(self):
        self.client.force_login(self.student)
        uri = reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid])
        response = self.client.get(uri)

        self.assertEqual(response.status_code, 403)

        
    def test_ticket_detail_update(self):
        uri = reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid])
        response = self.client.post(uri, {"status": StatusChoices.ACTIVE, "category": app_settings.TICKET_CATEGORIES.COMMENT, "priority": PriorityChoices.MEDIUM}, follow=True)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid]), 302)])
        self.assertContains(response, '<option value="10" selected>Active</option>')
        self.simple_ticket.refresh_from_db()
        self.assertEqual(self.simple_ticket.status, StatusChoices.ACTIVE)
        self.assertEqual(self.simple_ticket.comments.all().count(), 1)
        event = self.simple_ticket.comments.get(comment_type=CommentTypeChoices.EVENT)
        self.assertEqual(event.user, self.support)

        
    def test_comment_creation(self):
        self.assertEqual(Comment.objects.all().count(), 0)
        uri = reverse("helpme-api-create-comment", args=[self.simple_ticket.uuid])
        response = self.client.post(uri, {"content": "This is a test comment", "visibility": VisibilityChoices.REPORTERS}, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(reverse("helpme_admin:ticket-detail", args=[self.simple_ticket.uuid]), 302)])
        
        self.assertContains(response, "This is a test comment")
        self.assertEqual(Comment.objects.all().count(), 1)

        self.simple_ticket.refresh_from_db()
        self.assertEqual(self.simple_ticket.comments.all().count(), 1)


    def test_teams_view_support(self):
        uri = reverse("helpme_admin:team-list")
        response = self.client.get(uri)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "My Teams")
        self.assertContains(response, reverse("helpme_admin:team-detail", args=[self.support_team.uuid]))
        self.assertContains(response, '> Support </a>')


    def test_teams_view_admin(self):
        self.client.force_login(self.admin)
        uri = reverse("helpme_admin:team-list")
        response = self.client.get(uri)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Manage Teams")
        self.assertContains(response, reverse("helpme_admin:team-detail", args=[self.support_team.uuid]))
        self.assertContains(response, '> Support </a>')


    def test_teams_view_student(self):
        self.client.force_login(self.student)
        uri = reverse("helpme_admin:team-list")
        response = self.client.get(uri)

        # regular user should not have permission to view this page
        self.assertEqual(response.status_code, 403)


    def test_team_detail_support(self):
        uri = reverse("helpme_admin:team-detail", args=[self.support_team.uuid])
        response = self.client.get(uri)

        # lists team details but cannot update them 
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Support")
        self.assertContains(response, "Site(s):")
        self.assertContains(response, "example.com")
        self.assertContains(response, "Handles Categories:")
        self.assertContains(response, "Help")
        self.assertContains(response, "Members:")
        self.assertContains(response, "supportuser")
        self.assertNotContains(response, '<input type="submit" value="Update"')


    def test_team_detail_admin(self):
        self.client.force_login(self.admin)
        uri = reverse("helpme_admin:team-detail", args=[self.support_team.uuid])
        response = self.client.get(uri)

        # form to update team details
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<input type="text" name="name"')
        self.assertContains(response, 'checked="checked" name="categories"')
        self.assertContains(response, 'checked="checked" name="members"')
        self.assertContains(response, '<input type="submit" value="Update"')
        

    def test_team_detail_student(self):
        self.client.force_login(self.student)
        uri = reverse("helpme_admin:team-detail", args=[self.support_team.uuid])
        response = self.client.get(uri)

        # regular user should not have permission to view this page
        self.assertEqual(response.status_code, 403)


    def test_faq_view(self):
        uri = reverse("helpme:faq")
        response = self.client.get(uri)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Testing")
        self.assertContains(response, "What is 2 + 2?")
        self.assertContains(response, "4")


    def test_category_creation(self):
        uri = reverse("helpme-api-create-category")
        response = self.client.post(uri, {"category": "New!"}, follow=True)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(reverse("helpme_admin:faq-create"), 302)])
        self.assertContains(response, "New!")


    def test_question_creation(self):
        uri = reverse("helpme-api-create-question")
        response = self.client.post(uri, {"question": "What color is the sky?", "answer": "Blue", "category": self.basic_category.pk}, follow=True)
        
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.redirect_chain, [(reverse("helpme_admin:faq-create"), 302)])
        self.assertContains(response, "What color is the sky?")
        self.assertContains(response, "Blue")



from django.contrib import admin

from helpme.models import Ticket, Comment, Team, Question, Category

class TicketAdmin(admin.ModelAdmin):
    readonly_fields=('created', 'updated',)

admin.site.register(Ticket, TicketAdmin)
admin.site.register(Comment)
admin.site.register(Team)
admin.site.register(Category)
admin.site.register(Question)



import uuid 
import datetime

from multiselectfield import MultiSelectField

from django.conf import settings
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.db import models
from django.contrib.sites.models import Site

from .settings import app_settings


##################
# CHOICES
##################


class StatusChoices(models.IntegerChoices):
    OPEN = 1, _("Open")
    ACTIVE = 10, _("Active")
    HOLD = 20, _("Hold")
    CLOSED = 30, _("Closed")
    CANCELED = 40, _("Canceled")

class PriorityChoices(models.IntegerChoices):
    SUGGESTION = 1, _("Suggestion")
    LOW = 2, _("Low")
    MEDIUM = 3, _("Medium")
    HIGH = 4, _("High")
    URGENT = 5, _("Urgent")

class VisibilityChoices(models.IntegerChoices):
    REPORTERS = 1, _("Reporters")
    SUPPORT = 10, _("Support Handlers")
    DEVELOPERS = 15, _("Developers")
    SUPERVISORS = 20, _("Supervisors")

class CommentTypeChoices(models.IntegerChoices):
    MESSAGE = 0, _('Message')
    EVENT = 1, _('Event')


##################
# ABSTRACT MODELS
##################

class CreateUpdateModelBase(models.Model):
    created = models.DateTimeField("date created", auto_now_add=True)
    updated = models.DateTimeField("last updated", auto_now=True)

    class Meta:
        abstract = True

    def been_updated(self):
        return (self.updated - self.created) > datetime.timedelta(seconds=1)
    
##################
# MODELS
##################

class Category(models.Model):
    category = models.CharField(_("Category"), max_length=120)
    category_sites = models.ManyToManyField(Site, blank=True, related_name="categories")
    localization = models.JSONField(default=dict)
    global_category = models.BooleanField(default=False)
    category_excluded_sites = models.ManyToManyField(Site, blank=True, related_name="excluded_categories")

    class Meta:
        verbose_name_plural = "Categories"

    def __str__(self):
        return self.category


class Question(models.Model):
    question = models.CharField(_("Question"), max_length=120)
    answer = models.CharField(_("Answer"), max_length=255)
    category = models.ForeignKey(Category, null=True, blank=True, on_delete=models.SET_NULL, related_name="questions")
    sites = models.ManyToManyField(Site, blank=True, related_name="questions")
    localization = models.JSONField(default=dict)
    global_question = models.BooleanField(default=False)
    excluded_sites = models.ManyToManyField(Site, blank=True, related_name="excluded_questions")

    def __str__(self):
        return self.question
    

class Team(models.Model):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    name = models.CharField(_("Name"), max_length=30)
    global_team = models.BooleanField(default=False)
    members = models.ManyToManyField(settings.AUTH_USER_MODEL, blank=True)
    sites = models.ManyToManyField(Site)
    categories = MultiSelectField(_("Categories"), choices=app_settings.TICKET_CATEGORIES.choices)

    def __str__(self):
        return self.name
    

class Ticket(CreateUpdateModelBase):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    status = models.IntegerField(_("Status"), default=StatusChoices.OPEN, choices=StatusChoices.choices)
    priority = models.IntegerField(_("Priority"), default=PriorityChoices.MEDIUM, choices=PriorityChoices.choices)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.SET_NULL, related_name="support_tickets")
    user_meta = models.JSONField(blank=True, null=True, default=dict)
    category = models.IntegerField(_("Category"), choices=app_settings.TICKET_CATEGORIES.choices)
    teams = models.ManyToManyField(Team, blank=True, related_name="support_tickets")
    subject = models.CharField(_("Subject"), max_length=120)
    description = models.TextField(_("Message"))
    assigned_to = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.SET_NULL, related_name="assigned_tickets")
    dev_ticket = models.CharField(_("Developer Ticket"), max_length=30, blank=True, null=True)
    related_to = models.ManyToManyField("self", blank=True)
    site = models.ForeignKey(Site, default=1, on_delete=models.CASCADE)
    question = models.ForeignKey(Question, blank=True, null=True, on_delete=models.SET_NULL)

    class Meta:
        permissions = (
            ("see_support_tickets", "Access to support-level tickets"), 
            ("see_developer_tickets", "Access to developer-level tickets"), 
            ("see_all_tickets", "Access to all tickets"),
        ) 

    def __str__(self):
        return "{0} - {1}".format(self.user.username, self.subject)

    def log_history_event(self, action, user=None, notes=None, isotime=None):
        '''
        "history": [
            {
                "time": "2020-04-08T23:45:02.225609+00:00",
                "action": "created"
            }
        ],
        '''
        if not isotime:
            isotime = timezone.now().isoformat()           # '2019-12-18T22:27:28.222000+00:00'

        event = {'action':action, 'time':isotime}

        if user:
            event['user'] = user.pk
            event['username'] = user.username

        if notes:
            event['notes'] = notes

        self.history.append(event)

    
class Comment(CreateUpdateModelBase):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.SET_NULL)
    content = models.TextField(_("Content"))
    ticket = models.ForeignKey(Ticket, on_delete=models.CASCADE, related_name="comments")
    visibility = models.IntegerField(_("Visibility"), default=VisibilityChoices.REPORTERS, choices=VisibilityChoices.choices)
    comment_type = models.IntegerField(_("Type"), default=CommentTypeChoices.MESSAGE, choices=CommentTypeChoices.choices)

    def __str__(self):
        return "{0} - {1}".format(self.user.username, self.created)
    



from django.urls import path

from helpme.views import helpme_admin as views

app_name = "helpme_admin"

urlpatterns = [
    path("faq/create/", views.FAQCreateView.as_view(), name="faq-create"),
    path("tickets/", views.AdminSupportDashboardView.as_view(), name="dashboard"),
    path("ticket/<uuid:uuid>/", views.TicketDetailView.as_view(), name="ticket-detail"),
    path("teams/", views.TeamCreateView.as_view(), name="team-list"),
    path("teams/<uuid:uuid>/", views.TeamDetailView.as_view(), name="team-detail"),
]



# Generated by Django 3.1 on 2020-10-01 19:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0008_auto_20200930_2347'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ticket',
            options={'permissions': (('see_support_tickets', 'Access to support-level tickets'), ('see_developer_tickets', 'Access to developer-level tickets'), ('see_all_tickets', 'Access to all tickets'))},
        ),
    ]



# --------------------------------------------
# Copyright 2019, Grant Viklund
# @Author: Grant Viklund
# @Date:   2019-08-06 15:10:20
# --------------------------------------------

from rest_framework import serializers

from helpme.models import Ticket, Comment, Category, Question


class TicketSerializer(serializers.ModelSerializer):
    class Meta:
        model = Ticket
        fields = ('category', 'subject', 'description')
        

class CommentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Comment
        fields = ('content', 'visibility')


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ('category', 'category_sites', 'global_category', 'category_excluded_sites')


class QuestionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Question
        fields = ('question', 'answer', 'category', 'sites', 'global_question', 'excluded_sites')



# Generated by Django 3.1 on 2020-09-01 20:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('helpme', '0002_category_question'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='excluded_sites',
            field=models.ManyToManyField(blank=True, related_name='excluded_categories', to='sites.Site'),
        ),
        migrations.AddField(
            model_name='category',
            name='global_category',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='question',
            name='excluded_sites',
            field=models.ManyToManyField(blank=True, related_name='excluded_questions', to='sites.Site'),
        ),
        migrations.AddField(
            model_name='question',
            name='global_question',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='ticket',
            name='question',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='helpme.question'),
        ),
        migrations.AlterField(
            model_name='category',
            name='sites',
            field=models.ManyToManyField(related_name='categories', to='sites.Site'),
        ),
        migrations.AlterField(
            model_name='question',
            name='sites',
            field=models.ManyToManyField(blank=True, related_name='questions', to='sites.Site'),
        ),
    ]



# Generated by Django 3.1 on 2020-09-30 23:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('helpme', '0007_auto_20200912_0509'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'verbose_name_plural': 'Categories'},
        ),
    ]



# --------------------------------------------
# Copyright 2015-2020, Grant Viklund
# @Author: Grant Viklund
# --------------------------------------------

from os import path
from setuptools import setup, find_packages

from helpme.__version__ import VERSION

readme_file = path.join(path.dirname(path.abspath(__file__)), 'README.md')

try:
    from m2r import parse_from_file
    long_description = parse_from_file(readme_file)     # Convert the file to RST for PyPI
except ImportError:
    # m2r may not be installed in user environment
    with open(readme_file) as f:
        long_description = f.read()


package_metadata = {
    'name': 'django-help-me',
    'version': VERSION,
    'description': 'An app for providing a simple Help Desk & FAQ for users.',
    'long_description': long_description,
    'url': 'https://github.com/renderbox/django-help-me/',
    'author': 'Grant Viklund, Mackenzie Camisa',
    'author_email': 'renderbox@example.com',
    'license': 'MIT license',
    'classifiers': [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    'keywords': ['django', 'app'],
}

setup(
    **package_metadata,
    packages=find_packages(),
    package_data={'helpme': ['templates/helpme/*.html', 'templates/helpme/*.txt', 'templates/helpme/includes/*.html', 'static/js/support/*.js', 'locale/*/LC_MESSAGES/django.po']},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        'Django>=3.1, <3.2',
        'django-autoslug',
        'django-multiselectfield',
        'django-crispy-forms',
        'django-user-agents',
    ],
    extras_require={
        'dev': [
            'django-extensions',
            'django-allauth',
            'dj-database-url',
            'psycopg2-binary',
            'pylint',
            'djangorestframework',
            'pyyaml',
            'ua-parser',
            'user-agents',
        ],
        'test': [],
        'prod': [],
        'build': [
            'setuptools',
            'wheel',
            'twine',
            'm2r',
        ],
        'docs': [
            'coverage',
            'Sphinx',
            'sphinx-rtd-theme',
            'recommonmark',
            'rstcheck',
        ],
    }
)



from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class CategoryChoices(models.IntegerChoices):
    COMMENT = 1, _("Comment")
    SALES = 2, _("Sales")
    HELP = 3, _("Help")
    BUG = 4, _("Bug")

    
class AppSettings(object):

    def __init__(self, prefix):
        self.prefix = prefix

    def _setting(self, name, default):
        return getattr(settings, self.prefix + name, default)

    @property
    def TICKET_CATEGORIES(self):
        """ Category choices for a support request ticket """
        return self._setting('TICKET_CATEGORIES', CategoryChoices)


app_settings = AppSettings('HELPME_')



# Generated by Django 3.1 on 2020-08-27 19:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import multiselectfield.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('name', models.CharField(max_length=30)),
                ('global_team', models.BooleanField()),
                ('categories', multiselectfield.db.fields.MultiSelectField(choices=[(1, 'Comment'), (2, 'Sales'), (3, 'Help'), (4, 'Bug')], max_length=7)),
                ('members', models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL)),
                ('sites', models.ManyToManyField(default=1, to='sites.Site')),
            ],
        ),
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('status', models.IntegerField(choices=[(1, 'Open'), (10, 'Active'), (20, 'Hold'), (30, 'Closed'), (40, 'Canceled')], default=1, verbose_name='Status')),
                ('priority', models.IntegerField(choices=[(1, 'Suggestion'), (2, 'Low'), (3, 'Medium'), (4, 'High'), (5, 'Urgent')], default=3, verbose_name='Priority')),
                ('user_meta', models.JSONField(blank=True, default=dict, null=True)),
                ('category', models.IntegerField(choices=[(1, 'Comment'), (2, 'Sales'), (3, 'Help'), (4, 'Bug')], verbose_name='Category')),
                ('subject', models.CharField(max_length=120, verbose_name='Subject')),
                ('description', models.TextField(verbose_name='Description')),
                ('history', models.JSONField(blank=True, default=list, null=True)),
                ('dev_ticket', models.CharField(blank=True, max_length=30, null=True)),
                ('assigned_to', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='assigned_tickets', to=settings.AUTH_USER_MODEL)),
                ('related_to', models.ManyToManyField(blank=True, related_name='_ticket_related_to_+', to='helpme.Ticket')),
                ('site', models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
                ('teams', models.ManyToManyField(blank=True, related_name='support_tickets', to='helpme.Team')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='support_tickets', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'permissions': (('see-support-tickets', 'Access to support-level tickets'), ('see-developer-tickets', 'Access to developer-level tickets'), ('see-all-tickets', 'Access to all tickets')),
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='date created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='last updated')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('content', models.TextField(verbose_name='Content')),
                ('visibility', models.IntegerField(choices=[(1, 'Reporters'), (10, 'Support Handlers'), (15, 'Developers'), (20, 'Supervisors')], default=1, verbose_name='Visibility')),
                ('ticket', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comments', to='helpme.ticket')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
